<?php defined('SYSPATH') or die('No direct script access.');

class SupportImageSaver
{
    const savingStatusOk = true;

    private $_rootDir = DOCROOT;
    private $_subDir = 'support/img/';
    private $_fileExt = '.jpg';
    private $_thumbSuf = 'thumb_';

    private $_srcFilePath = null;
    private $_targetFilePath = null;
    private $_targetPreviewPath = null;
    private $_thumbHeight = 200;
    private $_thumbWidth = 200;
    private $_quality = 90;
    private $_previewCreated = false;

    function __construct($srcFilePath, $targetFileName) {
        $dir = $this->_rootDir . $this->_subDir;
        $file = $targetFileName . $this->_fileExt;

        $this->_srcFilePath = $srcFilePath;
        $this->_targetFilePath = $dir . $file;
        $this->_targetPreviewPath = $dir . $this->_thumbSuf . $file;
    }

    public function saveImage() {
        // TODO check size
        $srcImg = self::imageCreateFromFile($this->_srcFilePath);
        if ($srcImg == null) {
            return false;
        }
        $sx = imagesx($srcImg);
        $sy = imagesy($srcImg);
        $tarImg = imagecreatetruecolor($sx, $sy);

        imagefill($tarImg, 0, 0, imagecolorallocate($tarImg, 255, 255, 255));
        // imagealphablending($tarImg, TRUE);
        imagecopy($tarImg, $srcImg, 0, 0, 0, 0, $sx, $sy);
        $resultStatus = imagejpeg($tarImg, $this->_targetFilePath, $this->_quality);

        imagedestroy($srcImg);
        imagedestroy($tarImg);

        if ($resultStatus == self::savingStatusOk) {
            $thumbResultStatus = $this->createThumb();
            $this->_previewCreated = $thumbResultStatus;
        }

        return $resultStatus;
    }

    public function getPreviewBase64() {
        if ($this->_previewCreated) {
            return self::imageToBase64($this->_targetPreviewPath);
        }
        return '';
    }

    private function createThumb() {
        $sourcePath = $this->_targetFilePath;
        $targetPath = $this->_targetPreviewPath;

        $srcImg = imagecreatefromjpeg($sourcePath);
        $oldHeight = imageSX($srcImg);
        $oldWidth = imageSY($srcImg);
        $thumbW = $this->_thumbWidth;
        $thumbH = $this->_thumbHeight;

        // set thumb scale same as original scale
        if ($oldHeight > $oldWidth) {
            $thumbH = $oldWidth * ($thumbH / $oldHeight);
        }
        if ($oldHeight < $oldWidth) {
            $thumbW = $oldHeight * ($thumbW / $oldWidth);
        }

        $dstImg = imagecreatetruecolor($thumbW, $thumbH);
        imagecopyresampled($dstImg, $srcImg, 0, 0, 0, 0, $thumbW, $thumbH, $oldHeight, $oldWidth);
        $resultStatus = imagejpeg($dstImg, $targetPath);

        imagedestroy($dstImg);
        imagedestroy($srcImg);
        return $resultStatus;
    }

    // return resource of image by given filename
    private static function imageCreateFromFile($srcFileName) {
        $srcImg = null;
        $imageinfo = getimagesize($srcFileName);
        $format = $imageinfo['mime'];
        switch ($format) {
            case 'image/jpeg':
                $srcImg = imagecreatefromjpeg($srcFileName);
                break;
            case 'image/gif';
                $srcImg = imagecreatefromgif($srcFileName);
                break;
            case 'image/png':
                $srcImg = imagecreatefrompng($srcFileName);
                break;
        }
        return $srcImg;
    }

    private static function imageToBase64($path) {
        $type = pathinfo($path, PATHINFO_EXTENSION);
        $data = file_get_contents($path);
        $base64 = 'data:image/' . $type . ';base64,' . base64_encode($data);
        return $base64;
    }

}
